#!/usr/bin/env perl

use Test2::V0;
use TCOD;

is [ sort keys %TCOD:: ], [qw(
    AMBER
    AZURE
    Alignment
    BEGIN
    BKGND_ADD
    BKGND_ADDA
    BKGND_ADDALPHA
    BKGND_ALPH
    BKGND_ALPHA
    BKGND_BURN
    BKGND_COLOR_BURN
    BKGND_COLOR_DODGE
    BKGND_DARKEN
    BKGND_DEFAULT
    BKGND_LIGHTEN
    BKGND_MULTIPLY
    BKGND_NONE
    BKGND_OVERLAY
    BKGND_SCREEN
    BKGND_SET
    BLACK
    BLUE
    BRASS
    BackgroundFlag
    CELADON
    CENTER
    CHARTREUSE
    CHAR_ARROW2_E
    CHAR_ARROW2_N
    CHAR_ARROW2_S
    CHAR_ARROW2_W
    CHAR_ARROW_E
    CHAR_ARROW_N
    CHAR_ARROW_S
    CHAR_ARROW_W
    CHAR_BLOCK1
    CHAR_BLOCK2
    CHAR_BLOCK3
    CHAR_BULLET
    CHAR_BULLET_INV
    CHAR_BULLET_SQUARE
    CHAR_CENT
    CHAR_CHECKBOX_SET
    CHAR_CHECKBOX_UNSET
    CHAR_CLUB
    CHAR_COPYRIGHT
    CHAR_CROSS
    CHAR_CURRENCY
    CHAR_DARROW_H
    CHAR_DARROW_V
    CHAR_DCROSS
    CHAR_DHLINE
    CHAR_DIAMOND
    CHAR_DIVISION
    CHAR_DNE
    CHAR_DNW
    CHAR_DSE
    CHAR_DSW
    CHAR_DTEEE
    CHAR_DTEEN
    CHAR_DTEES
    CHAR_DTEEW
    CHAR_DVLINE
    CHAR_EXCLAM_DOUBLE
    CHAR_FEMALE
    CHAR_FUNCTION
    CHAR_GRADE
    CHAR_HALF
    CHAR_HEART
    CHAR_HLINE
    CHAR_LIGHT
    CHAR_MALE
    CHAR_MULTIPLICATION
    CHAR_NE
    CHAR_NOTE
    CHAR_NOTE_DOUBLE
    CHAR_NW
    CHAR_ONE_QUARTER
    CHAR_PILCROW
    CHAR_POUND
    CHAR_POW1
    CHAR_POW2
    CHAR_POW3
    CHAR_RADIO_SET
    CHAR_RADIO_UNSET
    CHAR_RESERVED
    CHAR_SE
    CHAR_SECTION
    CHAR_SMILIE
    CHAR_SMILIE_INV
    CHAR_SPADE
    CHAR_SUBP_DIAG
    CHAR_SUBP_E
    CHAR_SUBP_N
    CHAR_SUBP_NE
    CHAR_SUBP_NW
    CHAR_SUBP_SE
    CHAR_SUBP_SW
    CHAR_SW
    CHAR_TEEE
    CHAR_TEEN
    CHAR_TEES
    CHAR_TEEW
    CHAR_THREE_QUARTERS
    CHAR_UMLAUT
    CHAR_VLINE
    CHAR_YEN
    COLCTRL_1
    COLCTRL_2
    COLCTRL_3
    COLCTRL_4
    COLCTRL_5
    COLCTRL_BACK_RGB
    COLCTRL_FORE_RGB
    COLCTRL_NUMBER
    COLCTRL_STOP
    COPPER
    CRIMSON
    CYAN
    Char
    Color::
    ColorControl
    Console::
    DARKER_AMBER
    DARKER_AZURE
    DARKER_BLUE
    DARKER_CHARTREUSE
    DARKER_CRIMSON
    DARKER_CYAN
    DARKER_FLAME
    DARKER_FUCHSIA
    DARKER_GREEN
    DARKER_GREY
    DARKER_HAN
    DARKER_LIME
    DARKER_MAGENTA
    DARKER_ORANGE
    DARKER_PINK
    DARKER_PURPLE
    DARKER_RED
    DARKER_SEA
    DARKER_SEPIA
    DARKER_SKY
    DARKER_TURQUOISE
    DARKER_VIOLET
    DARKER_YELLOW
    DARKEST_AMBER
    DARKEST_AZURE
    DARKEST_BLUE
    DARKEST_CHARTREUSE
    DARKEST_CRIMSON
    DARKEST_CYAN
    DARKEST_FLAME
    DARKEST_FUCHSIA
    DARKEST_GREEN
    DARKEST_GREY
    DARKEST_HAN
    DARKEST_LIME
    DARKEST_MAGENTA
    DARKEST_ORANGE
    DARKEST_PINK
    DARKEST_PURPLE
    DARKEST_RED
    DARKEST_SEA
    DARKEST_SEPIA
    DARKEST_SKY
    DARKEST_TURQUOISE
    DARKEST_VIOLET
    DARKEST_YELLOW
    DARK_AMBER
    DARK_AZURE
    DARK_BLUE
    DARK_CHARTREUSE
    DARK_CRIMSON
    DARK_CYAN
    DARK_FLAME
    DARK_FUCHSIA
    DARK_GREEN
    DARK_GREY
    DARK_HAN
    DARK_LIME
    DARK_MAGENTA
    DARK_ORANGE
    DARK_PINK
    DARK_PURPLE
    DARK_RED
    DARK_SEA
    DARK_SEPIA
    DARK_SKY
    DARK_TURQUOISE
    DARK_VIOLET
    DARK_YELLOW
    DESATURATED_AMBER
    DESATURATED_AZURE
    DESATURATED_BLUE
    DESATURATED_CHARTREUSE
    DESATURATED_CRIMSON
    DESATURATED_CYAN
    DESATURATED_FLAME
    DESATURATED_FUCHSIA
    DESATURATED_GREEN
    DESATURATED_HAN
    DESATURATED_LIME
    DESATURATED_MAGENTA
    DESATURATED_ORANGE
    DESATURATED_PINK
    DESATURATED_PURPLE
    DESATURATED_RED
    DESATURATED_SEA
    DESATURATED_SKY
    DESATURATED_TURQUOISE
    DESATURATED_VIOLET
    DESATURATED_YELLOW
    DISTRIBUTION_GAUSSIAN
    DISTRIBUTION_GAUSSIAN_INVERSE
    DISTRIBUTION_GAUSSIAN_RANGE
    DISTRIBUTION_GAUSSIAN_RANGE_INVERSE
    DISTRIBUTION_LINEAR
    Dijkstra::
    Distribution
    EVENT_ANY
    EVENT_KEY
    EVENT_KEY_PRESS
    EVENT_KEY_RELEASE
    EVENT_MOUSE
    EVENT_MOUSE_MOVE
    EVENT_MOUSE_PRESS
    EVENT_MOUSE_RELEASE
    EVENT_NONE
    Event
    FLAME
    FONT_LAYOUT_ASCII_INCOL
    FONT_LAYOUT_ASCII_INROW
    FONT_LAYOUT_CP437
    FONT_LAYOUT_TCOD
    FONT_TYPE_GRAYSCALE
    FONT_TYPE_GREYSCALE
    FOV
    FOV_BASIC
    FOV_DIAMOND
    FOV_PERMISSIVE_0
    FOV_PERMISSIVE_1
    FOV_PERMISSIVE_2
    FOV_PERMISSIVE_3
    FOV_PERMISSIVE_4
    FOV_PERMISSIVE_5
    FOV_PERMISSIVE_6
    FOV_PERMISSIVE_7
    FOV_PERMISSIVE_8
    FOV_RESTRICTIVE
    FOV_SHADOW
    FUCHSIA
    FontFlag
    GOLD
    GREEN
    GREY
    HAN
    Image::
    K_0
    K_1
    K_2
    K_3
    K_4
    K_5
    K_6
    K_7
    K_8
    K_9
    K_ALT
    K_APPS
    K_BACKSPACE
    K_CAPSLOCK
    K_CHAR
    K_CONTROL
    K_DELETE
    K_DOWN
    K_END
    K_ENTER
    K_ESCAPE
    K_F1
    K_F10
    K_F11
    K_F12
    K_F2
    K_F3
    K_F4
    K_F5
    K_F6
    K_F7
    K_F8
    K_F9
    K_HOME
    K_INSERT
    K_KP0
    K_KP1
    K_KP2
    K_KP3
    K_KP4
    K_KP5
    K_KP6
    K_KP7
    K_KP8
    K_KP9
    K_KPADD
    K_KPDEC
    K_KPDIV
    K_KPENTER
    K_KPMUL
    K_KPSUB
    K_LEFT
    K_LWIN
    K_NONE
    K_NUMLOCK
    K_PAGEDOWN
    K_PAGEUP
    K_PAUSE
    K_PRINTSCREEN
    K_RIGHT
    K_RWIN
    K_SCROLLLOCK
    K_SHIFT
    K_SPACE
    K_TAB
    K_TEXT
    K_UP
    Key::
    Keycode
    LEFT
    LIGHTER_AMBER
    LIGHTER_AZURE
    LIGHTER_BLUE
    LIGHTER_CHARTREUSE
    LIGHTER_CRIMSON
    LIGHTER_CYAN
    LIGHTER_FLAME
    LIGHTER_FUCHSIA
    LIGHTER_GREEN
    LIGHTER_GREY
    LIGHTER_HAN
    LIGHTER_LIME
    LIGHTER_MAGENTA
    LIGHTER_ORANGE
    LIGHTER_PINK
    LIGHTER_PURPLE
    LIGHTER_RED
    LIGHTER_SEA
    LIGHTER_SEPIA
    LIGHTER_SKY
    LIGHTER_TURQUOISE
    LIGHTER_VIOLET
    LIGHTER_YELLOW
    LIGHTEST_AMBER
    LIGHTEST_AZURE
    LIGHTEST_BLUE
    LIGHTEST_CHARTREUSE
    LIGHTEST_CRIMSON
    LIGHTEST_CYAN
    LIGHTEST_FLAME
    LIGHTEST_FUCHSIA
    LIGHTEST_GREEN
    LIGHTEST_GREY
    LIGHTEST_HAN
    LIGHTEST_LIME
    LIGHTEST_MAGENTA
    LIGHTEST_ORANGE
    LIGHTEST_PINK
    LIGHTEST_PURPLE
    LIGHTEST_RED
    LIGHTEST_SEA
    LIGHTEST_SEPIA
    LIGHTEST_SKY
    LIGHTEST_TURQUOISE
    LIGHTEST_VIOLET
    LIGHTEST_YELLOW
    LIGHT_AMBER
    LIGHT_AZURE
    LIGHT_BLUE
    LIGHT_CHARTREUSE
    LIGHT_CRIMSON
    LIGHT_CYAN
    LIGHT_FLAME
    LIGHT_FUCHSIA
    LIGHT_GREEN
    LIGHT_GREY
    LIGHT_HAN
    LIGHT_LIME
    LIGHT_MAGENTA
    LIGHT_ORANGE
    LIGHT_PINK
    LIGHT_PURPLE
    LIGHT_RED
    LIGHT_SEA
    LIGHT_SEPIA
    LIGHT_SKY
    LIGHT_TURQUOISE
    LIGHT_VIOLET
    LIGHT_YELLOW
    LIME
    MAGENTA
    Map::
    Mouse::
    NB_FOV_ALGORITHMS
    NB_RENDERERS
    NOISE_DEFAULT
    NOISE_DEFAULT_HURST
    NOISE_DEFAULT_LACUNARITY
    NOISE_MAX_DIMENSIONS
    NOISE_MAX_OCTAVES
    NOISE_PERLIN
    NOISE_SIMPLEX
    NOISE_WAVELET
    Noise::
    NoiseType
    ORANGE
    PINK
    PINKEACH
    PURPLE
    Path::
    RED
    RENDERER_GLSL
    RENDERER_OPENGL
    RENDERER_OPENGL2
    RENDERER_SDL
    RENDERER_SDL2
    RIGHT
    RNG_CMWC
    RNG_MT
    Random::
    RandomAlgo
    Renderer
    SEA
    SEPIA
    SILVER
    SKY
    Sys::
    TURQUOISE
    VERSION
    VIOLET
    WHITE
    YELLOW
    __ANON__
    import
)] => 'No unexpected methods in TCOD namespace';

done_testing;
